import React, { useEffect, useRef } from "react";
import { Pane } from "./Pane";
import { Dividers } from "./Dividers";
import AnimationFrame from "./AnimationFrame";
import { CHILD_ABOVE, CHILD_BELOW, CHILD_LEFT, CHILD_RIGHT, ROW, SW, NE, SE, NW } from "../reducer/constants";
Layout.defaultProps = {
  iframeSafe: true
};
export function Layout(props) {
  var propsRef = useRef();
  propsRef.current = props;
  useEffect(function () {
    var props = propsRef.current;
    var animationFrame = new AnimationFrame();
    var setSize = props.actions.setSize;
    var onMouseMove = animationFrame.throttle(function (e) {
      var actions = props.actions,
          subdivide = props.store;
      var clientX = e.clientX,
          clientY = e.clientY;

      if (subdivide.dividerDown) {
        e.preventDefault();
        var divider = subdivide.dividerDown;
        var beforePaneId = divider.beforePaneId,
            afterPaneId = divider.afterPaneId,
            direction = divider.direction,
            parentSize = divider.parentSize,
            startX = divider.startX,
            startY = divider.startY;
        var delta = direction === ROW ? clientX - startX : clientY - startY;
        var deltaRatio = delta / parentSize;
        var afterRatio = divider.afterRatio - deltaRatio;
        var beforeRatio = divider.beforeRatio + deltaRatio;

        if (beforeRatio * parentSize > 20 && afterRatio * parentSize > 20) {
          actions.setSplitRatio(beforePaneId, beforeRatio);
          actions.setSplitRatio(afterPaneId, afterRatio);
        }
      }

      if (subdivide.cornerDown) {
        var pane = subdivide.cornerDown;
        var split = actions.split;
        var width = pane.width,
            height = pane.height,
            left = pane.left,
            top = pane.top,
            id = pane.id,
            corner = pane.corner;

        if (clientX > left && clientX < left + width && clientY > top && clientY < top + height) {
          if (corner === SW) {
            if (clientX - left > 25) {
              split(id, CHILD_LEFT, clientX, clientY);
            } else if (top + height - clientY > 25) {
              split(id, CHILD_BELOW, clientX, clientY);
            }
          }

          if (corner === NE) {
            if (left + width - clientX > 25) {
              split(id, CHILD_RIGHT, clientX, clientY);
            } else if (clientY - top > 25) {
              split(id, CHILD_ABOVE, clientX, clientY);
            }
          }

          if (corner === SE) {
            if (left + width - clientX > 25) {
              split(id, CHILD_RIGHT, clientX, clientY);
            } else if (top + height - clientY > 25) {
              split(id, CHILD_BELOW, clientX, clientY);
            }
          }

          if (corner === NW) {
            if (clientX - left > 25) {
              split(id, CHILD_LEFT, clientX, clientY);
            } else if (clientY - top > 25) {
              split(id, CHILD_ABOVE, clientX, clientY);
            }
          }
        }
      }
    }, []);

    function onMouseUp() {
      var actions = props.actions,
          subdivide = props.store;

      if (subdivide.dividerDown) {
        actions.setDividerDown(undefined);
      } // give pane onMouseUp a chance to fire


      setTimeout(function () {
        if (subdivide.cornerDown) {
          actions.setCornerDown(undefined);
        }
      }, 10);
    }

    window.addEventListener("resize", function () {
      setSize(window.innerWidth, window.innerHeight);
    });
    document.addEventListener("mouseup", onMouseUp);
    document.addEventListener("mousemove", onMouseMove);
    setSize(window.innerWidth, window.innerHeight);
    return animationFrame.stop;
  }, []);
  var subdivide = props.store,
      actions = props.actions,
      DefaultComponent = props.DefaultComponent,
      iframeSafe = props.iframeSafe;
  var panes;

  if (iframeSafe) {
    panes = subdivide.allPanesIdsEver.map(function (id) {
      var pane = subdivide.panes[id];
      return React.createElement(Pane, {
        subdivide: subdivide,
        pane: pane,
        actions: actions,
        key: "pane" + id,
        DefaultComponent: DefaultComponent
      });
    });
  } else {
    panes = subdivide.panes.filter(function (pane) {
      return !pane.isGroup;
    }).map(function (pane) {
      return React.createElement(Pane, {
        subdivide: subdivide,
        pane: pane,
        actions: actions,
        key: pane.id,
        DefaultComponent: DefaultComponent
      });
    });
  }

  return React.createElement("div", null, panes, React.createElement(Dividers, {
    dividers: subdivide.dividers,
    subdivide: subdivide,
    actions: actions
  }));
}