import React from "react";
import { Triangle } from "./Triangle";
import CornerOverlay from "./CornerOverlay";
import { PaneProvider } from "./usePane";
import { NE, SW, SE, NW } from "../reducer/constants";

function getStyles(_ref) {
  var width = _ref.width,
      height = _ref.height,
      top = _ref.top,
      left = _ref.left;
  return {
    pane: {
      position: "absolute",
      width: width + "px",
      height: height + "px",
      top: top + "px",
      left: left + "px",
      overflow: "hidden"
    }
  };
}

export function Pane(_ref2) {
  var pane = _ref2.pane,
      subdivide = _ref2.subdivide,
      actions = _ref2.actions,
      DefaultComponent = _ref2.DefaultComponent;

  function onMouseUp() {
    //Note this on mouse up happens after subdivide on mouse up
    var join = actions.join;
    if (!subdivide.cornerDown) return;
    var cornerDownId = subdivide.cornerDown.id;

    if (pane.joinDirection) {
      join(cornerDownId, pane.id);
      actions.setCornerDown(undefined);
    }
  }

  if (pane === undefined) {
    return React.createElement("div", {
      style: {
        visibility: "hidden"
      }
    });
  }

  if (pane.isGroup) {
    return null;
  }

  var styles = getStyles(pane);
  return React.createElement(PaneProvider, {
    value: pane
  }, React.createElement("div", {
    style: styles.pane,
    onMouseUp: onMouseUp
  }, React.createElement(DefaultComponent, {
    subdividePane: pane,
    subdivideActions: actions,
    subdivide: subdivide
  }), React.createElement(CornerOverlay, {
    pane: pane,
    subdivide: subdivide
  }), React.createElement(Triangle, {
    corner: SW,
    color: "#dadadf",
    size: 42,
    subdivide: subdivide,
    pane: pane,
    actions: actions
  }), React.createElement(Triangle, {
    corner: NE,
    color: "#dadadf",
    size: 42,
    subdivide: subdivide,
    pane: pane,
    actions: actions
  }), React.createElement(Triangle, {
    corner: NW,
    color: "#dadadf",
    size: 42,
    subdivide: subdivide,
    pane: pane,
    actions: actions
  }), React.createElement(Triangle, {
    corner: SE,
    color: "#dadadf",
    size: 42,
    subdivide: subdivide,
    pane: pane,
    actions: actions
  })));
}